'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''
import math
import data
import os
import sys
#import ctypes as c

import pyglet
from pyglet import gl
from pyglet.window import key, mouse
#from pyglet.media import have_avbin

import pymunk
from pymunk import Vec2d, SlideJoint, PinJoint

from random import random, randint

import splinedemo
from splinedemo import spline

import squirtle

from objects import Scenery, Obstacle

def main():
    #print "Hello from your game's main()"
    #print data.load('sample.txt').read()
    w = HelloWorldWindow()
    pyglet.app.run()

def flatten(lst):
    for elem in lst:
        if type(elem) in (tuple, list):
            for i in flatten(elem):
                yield i
        else:
            yield elem

class HelloWorldWindow(pyglet.window.Window):
    lid = None
    cat = None
    static_lines = None
    winX = 640
    winY = 480
    show_fps = False
    truck_svg = None
    cat_svg = None
    street_svg = None
    cloud_svg = None
    hill_svg = None
    sidewalk = None
    grass = None
    time = 0
    obstacles = []
    scenery = []
    tuna = []
    show_ghost = []
    lives = 9
    gui = None
    score = 0
    links = []

    
    def __init__(self):
        super(HelloWorldWindow, self).__init__(width=self.winX, height=self.winY)
        squirtle.setup_gl()
        pymunk.init_pymunk()
        #gl.Disable(gl.GL_DEPTH_TEST)
        #gl.glFrontFace(gl.GL_CW)

        self.truck_svg = squirtle.SVG(data.filepath('truck.svg'), anchor_x='right', anchor_y='top')
        self.cat_svg = squirtle.SVG(data.filepath('cat.svg'), anchor_x='center', anchor_y='center')
        self.ghost_svg = squirtle.SVG(data.filepath('ghost.svg'), anchor_x='center', anchor_y='center')
        self.street_svg = squirtle.SVG(data.filepath('street.svg'), anchor_x='left', anchor_y='bottom')
        self.cloud_svg = squirtle.SVG(data.filepath('cloud.svg'), anchor_x='center', anchor_y='center')
        self.spark_svg = squirtle.SVG(data.filepath('spark.svg'), anchor_x='center', anchor_y='center')
        self.mailbox_svg = squirtle.SVG(data.filepath('mailbox.svg'), anchor_x='center', anchor_y='center')
        self.hydrant_svg = squirtle.SVG(data.filepath('hydrant.svg'), anchor_x='center', anchor_y='center')
        self.hill_svg = squirtle.SVG(data.filepath('hill.svg'), anchor_x='center', anchor_y='center')
        self.tuna_svg = squirtle.SVG(data.filepath('tuna.svg'), anchor_x='center', anchor_y='center')


        sidewalk_images = [pyglet.image.load(data.filepath('sidewalk08.png')),
                           pyglet.image.load(data.filepath('sidewalk07.png')),
                           pyglet.image.load(data.filepath('sidewalk06.png')),
                           pyglet.image.load(data.filepath('sidewalk05.png')),
                           pyglet.image.load(data.filepath('sidewalk04.png')),
                           pyglet.image.load(data.filepath('sidewalk03.png')),
                           pyglet.image.load(data.filepath('sidewalk02.png')),
                           pyglet.image.load(data.filepath('sidewalk01.png'))]
                           
                           
        self.sidewalk = pyglet.sprite.Sprite(
            pyglet.image.Animation.from_image_sequence(sidewalk_images, 0.005)
        );
        self.grass = pyglet.sprite.Sprite(pyglet.image.load(data.filepath('grass.png')))
        self.grass.y = 150
        #pyglet.gl.glClearColor(*white)
        #self.set_mouse_visible(False)
        self.set_vsync(False)
        gl.glClearColor(0.7, 0.7, 1.0, 1.0)
        pyglet.clock.schedule_interval(self.update, 1/60.0)
        self.fps_display = pyglet.clock.ClockDisplay()
        
        self.set_caption("StreetSurf")
        self.labels = [pyglet.text.Label('Q: Quit'),
        				pyglet.text.Label('R: Reset'),
        				pyglet.text.Label('UP: Jump'),
        				pyglet.text.Label('RIGHT / LEFT: Rotate')]
        for i in range(len(self.labels)):
        	self.labels[i].x = 450
        	self.labels[i].y = 450 - i * 20
        self.gui = [pyglet.text.Label('Lives: 9'),
                    pyglet.text.Label('Tuna: 0')]
        for i in range(len(self.gui)):
            self.gui[i].x = 450
            self.gui[i].y = 300 - i * 20
        #self.create_menu()
        self.create_world()
        
    #def create_menu():
        
        
    def create_world(self):
        self.obstacles = []
        self.tuna = []
        self.space = pymunk.Space()
        self.space.gravity = Vec2d(0., -900)

        self.sparks = pymunk.Space()
        self.sparks.gravity = Vec2d(0., -900)
        
        static_body = pymunk.Body(pymunk.inf, pymunk.inf)
        self.static_line = pymunk.Segment(static_body, Vec2d(-900,50), Vec2d(self.winX + 900,50),1)
        self.static_line.friction = 0.99
        self.static_line.surface_velocity = Vec2d(1000,0)
        self.space.add_static(self.static_line)
        self.sparks.add_static(self.static_line)
        
        self.links = []
        y = 0
        x = 0
        size = 0.1
        spread = 20
        body = pymunk.Body(pymunk.inf, pymunk.inf)
        body.position = Vec2d(150, 300)
        shape = pymunk.Circle(body, size, (0,0))
        shape.friction = 0.6
        self.space.add_static(shape)
        self.links.append(body)
        
        for x in range(13):
            mass = 0.01
            moment = pymunk.moment_for_circle(mass, 0, size, (0,0))
            body = pymunk.Body(mass, moment)
            body.position = Vec2d(150 + x * spread, 300 + y * spread)
            shape = pymunk.Circle(body, size, (0,0))
            shape.friction = 0.4
            self.space.add(body, shape)
            self.links.append(body)
        
        #x += 1
        mass = 50.0
        points = [Vec2d(30,0), Vec2d(30,-5), Vec2d(15,-10), Vec2d(3,-12), Vec2d(-3,-12), Vec2d(-15,-10), Vec2d(-30,-5), Vec2d(-30, 0)]#, Vec2d(-30,5), Vec2d(-25,25), Vec2d(-5,30), Vec2d(5,30),Vec2d(25,25)]
        #moment = pymunk.moment_for_circle(mass, 0, size, (0,0))
        #moment = pymunk.moment_for_poly(mass, points, (25,0))
        moment = pymunk.moment_for_poly(mass, points, (0,0))
        body = pymunk.Body(mass, moment)
        body.position = Vec2d(150 + x * spread, 300 + y * spread)
        shape = pymunk.Poly(body, points, (0,0))
        shape.friction = 0.3
        #shape.friction = 0.0
        shape.elasticity = 0.05
        self.space.add(body, shape)
        self.links.append(body)
        self.lid = shape
        
        #points = [Vec2d(-20,1), Vec2d(-10,10), Vec2d(10,10), Vec2d(20,1)]
        #body = pymunk.Body(0.0, 0.0)
        #body.position = Vec2d(50 + x * spread, 200 + y * spread)
        #shape = pymunk.Poly(body, points, (0,0))
        #shape.friction = 0.0
        #shape.elasticity = 0.0
        #self.space.add(body, shape)
        #self.cat = shape
        
        self.space.add( SlideJoint(self.links[0], self.links[1], Vec2d(0,0), Vec2d(0,0),0,0))
        for i in xrange(2,len(self.links) - 1):
            self.space.add( SlideJoint(self.links[i-1], self.links[i], Vec2d(0,0), Vec2d(0,0), 0, 25) )
        self.space.add( SlideJoint(self.links[-1], self.links[-2], Vec2d(-20,5), Vec2d(0,0), 0, 0) )
        
        #for i in xrange(1,len(links) - 1):
        #    self.space.add( PinJoint(links[i-1], links[i], Vec2d(0,0), Vec2d(0,0)) )
        #self.space.add( PinJoint(links[-1], links[-2], Vec2d(-30,0), Vec2d(0,0)) )

    def update(self, dt):
        self.time += dt
        #print self.time
        steps = 30
        for x in range(steps):
            self.space.step(1/60./steps)
        steps = 10
        for x in range(steps):
            self.sparks.step(1/60./steps)

    def addTuna(self):
        mass = 3
        points = [Vec2d(-8.5, -5), Vec2d(-8.5, 5), Vec2d(8.5, 5), Vec2d(8.5, -5)]
        moment = pymunk.moment_for_poly(mass, points, Vec2d(0,0))
        body = pymunk.Body(mass, moment)
        shape = pymunk.Poly(body, points, (0,0))
        shape.friction = 0.3
        shape.elasticity = 0.5
        shape.body.apply_impulse((randint(500,2000),0), (0,0))
        shape.body.position = Vec2d(100,430)
        self.tuna.append(shape)
        self.space.add(shape, shape.body)

    def addCloud(self):
        self.scenery.append(Scenery(self.cloud_svg, randint(10,50), 108, 48, randint(300,500), self.time))
        self.scenery = [i for i in self.scenery if i.x < 640 + i.width]
        self.scenery.sort(cmp=lambda a,b:b.depth - a.depth)

    def addHill(self):
        depth = randint(3,8)
        self.scenery.append(Scenery(self.hill_svg, depth, 50, 25, 160 + depth * 8, self.time))
        self.scenery = [i for i in self.scenery if i.x < 640 + i.width]
        self.scenery.sort(cmp=lambda a,b:b.depth - a.depth)

    def addHydrant(self):
        self.obstacles = [i for i in self.obstacles if i.x < 700]
        points = [Vec2d(-20,-40), Vec2d(-20, 40), Vec2d(20,40), Vec2d(20,-40)]
        o = Obstacle(self.hydrant_svg, 1, 0, 0, 100, self.time, points)
        self.obstacles.append(o)
        self.space.add(o.shape.body, o.shape)

    def addMailBox(self):
        self.obstacles = [i for i in self.obstacles if i.x < 700]
        points = [Vec2d(-57,-95), Vec2d(-57, 47), Vec2d(-25, 65), Vec2d(0,95), Vec2d(25,65), Vec2d(57,47), Vec2d(57,-95)]
        o = Obstacle(self.mailbox_svg, 1, 0, 0, 100, self.time, points)
        self.obstacles.append(o)
        self.space.add(o.shape.body, o.shape)

    def on_key_press(self, symbol, modifiers):
        if symbol == key.ESCAPE or symbol == key.Q:
            # Quit
            pyglet.app.exit()
        elif symbol == key.UP:
            # Jump
            if (self.lid.body.position.y < 90):
                f = 20000
                self.lid.body.apply_impulse((0,f),(randint(-1,1),0))
        elif symbol == key.RIGHT:
            # Rotate Clockwise
            f = 1500
            self.lid.body.apply_impulse((0,f),(-20,0))
            self.lid.body.apply_impulse((0,-f),(20,0))
        elif symbol == key.LEFT:
            # Rotate Counter-Clockwise
            f = 1500
            self.lid.body.apply_impulse((0,f),(20,0))
            self.lid.body.apply_impulse((0,-f),(-20,0))
        elif symbol == key.R:
            # Reset
            print "---- Resetting ----"
            self.create_world()
        elif symbol == key.F:
            self.show_fps = not self.show_fps
        #elif symbol == key.D:
        #    "do nothing"
            #glViewport(0,0,self.winX, self.winY)
            #glMatrixMode(GL_PROJECTION)
            #glLoadIdentity()
            #glOrtho(-zoom, self.winX, -zoom, self.winY, -1, 1)
            #glMatrixMode(GL_MODELVIEW)
        
            
    def cullSpace(self, shape):
        if shape.body.position.x > self.winX + 100 or shape.body.position.y < -100 or shape.body.position.y > self.winY + 100:
            self.space.remove(shape)
            #space.remove(shape.body)

    def cullSpark(self, shape):
        if shape.body.position.x > self.winX + 10:
            self.sparks.remove(shape)
            #space.remove(shape.body)
            
    def on_draw(self):
        if random() < 0.005:
            self.addCloud()
        if random() < 0.05:
            self.addHill()
        if random() < 0.005:
            self.addTuna()
        if random() < 0.005:
            self.addHydrant()
        if random() < 0.001:
            self.addMailBox()
        map(self.cullSpace, self.space.shapes)
        map(self.cullSpark, self.sparks.shapes)
        self.clear()

        self.b = pyglet.graphics.Batch()

        #for shape in self.space.static_shapes:
        #    if isinstance(shape, pymunk.Segment):
        #        body = shape.body
        #        pv1 = body.position + shape.a.rotated(math.degrees(body.angle))
        #        pv2 = body.position + shape.b.rotated(math.degrees(body.angle))
        #        self.b.add(2, gl.GL_LINES, None,
        #                ('v2f', (pv1.x, pv1.y, pv2.x, pv2.y)),
        #                ('c4f', (.85, .85, .85, 1.0) * 2))
        
        cs = []
        for shape in self.space.shapes:
        #    if isinstance(shape, pymunk.Poly):
        #        ps = shape.get_points()
        #        ps = [ps[0]] + ps + [ps[0], ps[0]]
        #        xs = []
        #        for p in ps:
        #            xs.append(p.x)
        #            xs.append(p.y)
        #        self.b.add(len(ps), gl.GL_TRIANGLES, None,
        #                ('v2f', xs),
        #                ('c4f', (0.0, 1.0, 0.0, 1.0) * len(ps)))
            if isinstance(shape, pymunk.Circle):
                p = shape.body.position
                cs.append(p)
        cs = [cs[0]] + cs + [cs[-1]]
        resolution = 16
        xs = list(flatten(spline(cs, resolution)))
        numPoints = (len(cs) - 3) * resolution + 1
        #b.add(numPoints, GL_LINE_STRIP, None,
        self.b.add(numPoints, gl.GL_LINE_STRIP, None,
                ('v2f', xs),
        #        ('c4f', (1.0, 1.0, 1.0, 1.0) * numPoints))
                ('c4f', (.85, .85, .85, 1.0) * numPoints))
        gl.glPointSize(2)
        
        self.grass.draw()
        self.street_svg.draw(-1700 + (self.time * 500) % 1700,50,angle=0,scale=1)
        #self.cloud_svg.draw(-110 + (self.time * 100) % 800, 300, angle=0, scale=1)
        for i in self.scenery:
            i.draw(self.time)
        self.truck_svg.draw(150, 500, angle = 0, scale = 1)
        map(self.checkCollisions, self.space.arbiters)
        self.b.draw()
        self.sidewalk.draw()
        if (self.lid.body.position.x > 650 or self.lid.body.position.y < 50):
            self.lives -= 1
            if (self.lives == 0):
                pyglet.app.exit()
            else:
                'no-op'
                self.gui[0].text = "Lives: " + str(self.lives)
                self.clear()
                self.create_world()
                self.show_ghost.insert(0,self.time)
        
        self.cat_svg.draw(self.lid.body.position.x, self.lid.body.position.y, angle=math.degrees(self.lid.body.angle), scale=0.8)
        for shape in self.sparks.shapes:
            if isinstance(shape, pymunk.Circle):
                p = shape.body.position
                self.spark_svg.draw(p.x, p.y)
                #self.b.add(1, gl.GL_POINTS, None,
                #        ('v2f', (p.x, p.y)),
                #        ('c4f', (1.0, 0.5, 0.0, 1.0)))
        for o in self.obstacles:
            o.draw()
        if (self.show_ghost):
            for i in self.show_ghost:
                x = self.lid.body.position.x + (self.time - i) * 30
                y = self.lid.body.position.y + (self.time - i) * 50
                if x > 660 or y > 510:
                    self.show_ghost.remove(i)
                    continue
                self.ghost_svg.draw(x,y,angle = math.sin((self.time - i)))
        self.tuna = [i for i in self.tuna if i.body.position.x < 650]
        for t in self.tuna:
            self.tuna_svg.draw(t.body.position.x, t.body.position.y, angle = math.degrees(t.body.angle))
            
        if (self.show_fps):
            self.fps_display.draw()
        for label in self.labels:
        	label.draw()
        for label in self.gui:
            label.draw()

    def checkCollisions(self, arb):
        if (arb.b == self.lid and arb.a == self.static_line):
            if abs(self.lid.body.angle) > math.pi * 0.95:
                self.lives -= 1
                if (self.lives > 0):
                    'no-op'
                    self.gui[0].text = "Lives: " + str(self.lives)
                    self.clear()
                    self.create_world()
                    self.show_ghost.insert(0,self.time)
                else:
                    pyglet.app.exit()
                
            for contact in arb.contacts:
                #if (contact.jn_acc < -0.07):
                #    print "OOF!", contact.jn_acc
                if (random() < 0.1):
                    for i in range(randint(0,3)):
                        size = 1
                        mass = 0.01
                        moment = pymunk.moment_for_circle(mass, 0, size, (0,0))
                        body = pymunk.Body(mass, moment)
                        body.position = Vec2d(contact.position.x, contact.position.y)
                        shape = pymunk.Circle(body, size, (0,0))
                        shape.friction = 0.9
                        shape.elasticity = 0.9
                        f = 5
                        body.apply_impulse((f + random() * f, 0.2 * f + 0.2 * random() * f),(20,0))
                        self.sparks.add(body, shape)
                        #self.space.add(body, shape)
                        #self.b.add(1, GL_POINTS, None,
                        #    ('v2f', (contact.position.x, contact.position.y)),
                        #    ('c4f', (1.0,0.5,0.0,1.0)))
        elif (arb.b == self.lid and arb.a in self.tuna):
            self.score += 1
            self.gui[1].text = "Tuna: " + str(self.score)
            self.tuna.remove(arb.a)
            self.space.remove(arb.a)
        #elif (arb.a == self.lid and arb.b in self.tuna):
        #    self.score += 1
        #    self.gui[1].text = "Tuna: " + str(self.score)
        #    self.tuna.remove(arb.b)
        #    self.space.remove(arb.b)
