import math
import pymunk
from pymunk import Vec2d


class Scenery:
    svg = None
    depth = 0
    width = 0
    height = 0
    y = 0
    x = 0
    start_time = 0
    
    def __init__(self, svg, depth, width, height, y, start_time):
        self.svg = svg
        self.depth = depth
        self.width = width
        self.height = height
        self.y = y
        self.x = 0
        self.start_time = start_time
        
    def draw(self, time):
        self.x = (time - self.start_time) * 800 / self.depth
        self.svg.draw(self.x,
                      self.y, angle=0, scale=1)

class Obstacle(Scenery):
    shape = None

    def __init__(self, svg, depth, width, height, y, start_time, points):
        Scenery.__init__(self, svg, depth, width, height,
                                       y, start_time)
        mass = 1000.0
        moment = pymunk.moment_for_poly(mass, points, Vec2d(0,-y/2))
        body = pymunk.Body(mass, moment)
        self.shape = pymunk.Poly(body, points, (0,0))
        self.shape.friction = 0.9
        self.shape.elasticity = 0.2
        self.shape.body.apply_impulse((1000000,0),(0,0))
        body.position = Vec2d(-5, y)
        self.shape.get_points()

    def draw(self):
        'no-op'
        self.svg.draw(self.shape.body.position.x,
                      self.shape.body.position.y,
                      angle=math.degrees(self.shape.body.angle),
                      scale=1)
