
"""Code for 10 spline functions.

# Copyright (c) 1996 Steve Noskowicz

Converted to Python by LJO 20080703
under the same license - I assume public domain - as the original code.

See the original article at the the end of this file.
"""

POINTRESULTS = True

def spline(v, n):
    """ v = array of points (equal sized tuples), 
        n = number of values to be produced per segment
    """
    e = 0
    m = len(v)
    dim = len(v[0])
    if e == 0:
        m -= 3

    dims = []
    for i in xrange(dim):
        vi = [p[i] for p in v]
        dims.append(vi)
        if e:
            vi.insert(0, 2 * vi[0] - vi[1])
            vi.append(2 * vi[-1] - vi[-2]); 
            vi.append(2 * vi[-1] - vi[-2]);
    
    start = 1
    end = m + 1
    step = 1
    js = 0
    d = 2.0
    t1 = -1 / d;  t2 = +3 / d;  t3 = -3 / d;  t4 = +1 / d
    u1 = +2 / d;  u2 = -5 / d;  u3 = +4 / d;  u4 = -1 / d
    v1 = -1 / d;  v2 = 0;       v3 = +1 / d;  v4 = 0
    w1 = 0;       w2 = +2 / d;  w3 = 0;       w4 = 0

    if POINTRESULTS:
        results = []
    else:
        results = [[] for d in dims]
    n0 = 0
    for i in xrange(start, end, step):
        if POINTRESULTS:
            coef = []
            for d in dims:
                va = d[i-1]; vb = d[i]; vc = d[i+1]; vd = d[i+2]
                coef.append((
                    t1 * va + t2 * vb + t3 * vc + t4 * vd,
                    u1 * va + u2 * vb + u3 * vc + u4 * vd,
                    v1 * va + v2 * vb + v3 * vc + v4 * vd,
                    w1 * va + w2 * vb + w3 * vc + w4 * vd))
            for j in xrange(n0, n + 1):          
                t = float(j + js) / n       
                results.append(
                    [((co[0] * t + co[1]) * t + co[2]) * t + co[3] for co in coef])
        else:
            for j, d in enumerate(dims):
                va = d[i-1]; vb = d[i]; vc = d[i+1]; vd = d[i+2]
                ca = t1 * va + t2 * vb + t3 * vc + t4 * vd
                cb = u1 * va + u2 * vb + u3 * vc + u4 * vd
                cc = v1 * va + v2 * vb + v3 * vc + v4 * vd
                cd = w1 * va + w2 * vb + w3 * vc + w4 * vd
                dst = results[j]
                for j in xrange(n):          
                    t = float(j + js) / n       
                    dst.append(((ca * t + cb) * t + cc) * t + cd)
        if i == start:
            n0 = 1
    return results
